 /* @(#) original: samples/c/amqswlm0.c, pcode, p510b, p510-06-000925   */
 /***********************************************************************/
 /*                                                                     */
 /* Program name:	RUUDEXI2                                            */
 /* Author			Ruud van Zundert April 2001                         */
 /*                                                                     */
 /* Description:	Cluster workload exit originally based on the       */
 /*					supplied sample. It only writes messages to the     */
 /*					queue manager with the highest network priority     */
 /*					on its cluster receiver channel.                    */
 /*					If there are several queue managers with the same   */
 /*					priority, then workload is distributed amongst them */
 /*					in a round-robin fashion.                           */
 /*					If there is no appropriate target destination, it   */
 /*					sets return code 2188 (rejected by cluster exit).   */
 /*                                                                     */
 /* Dependencies	In order to keep track of the last chosen           */
 /*                 destination, the exit uses the first byte of the    */
 /*                 'ExitUserArea' to store it.                         */
 /*                                                                     */
 /* Prerequisite	You must alter the sending queue manager by issuing */
 /*    ALTER QMGR CLWLEXIT('C:\???\priority(clwlFunction)')				*/
 /*                 and then restart the qmgr.                          */
 /*																		*/
 /* Tested			Under Windows 2000 Service Pack 1 and MQSeries for  */
 /*					Windows NT V5.1 CSD 6. Compiled under MS Visual C++ */
 /*					as a multithreaded DLL.	You'll need to include the 	*/
 /*					MQUTL.LIB library.									*/
 /*																		*/
 /* Warrenty		None. This is supplied ASIS, and you alone are      */
 /*					responsible if anything goes wrong.					*/
 /***********************************************************************/
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
 #include <cmqc.h>
 #include <cmqxc.h>
 #include <cmqcfc.h>
 
 void MQENTRY clwlFunction ( MQWXP *parms )
 {
   MQLONG   index,j;
   MQLONG   GoodDestination, AtLeastOne = 0;
   MQWDR  * qmgr;
   MQWQR  * queue;
   MQCD   * cd;
   MQLONG   CompCode = MQCC_OK;
   MQLONG   Reason   = MQRC_NONE;
   MQPTR    NextRecord;
   char		tmpbuf[10],tmpbuf1[1];
   long		Highest = 0, Highest_Sel = 0;

   /******************************************************************/
   /* Set the ExitResponse2 to indicate this exit is compatable with */
   /* a dynamic cache (ie it uses MQXCLWLN to navigate along lists   */
   /* of cluster records and channel definitions)                    */
   /******************************************************************/
   parms->ExitResponse2 = MQXR2_DYNAMIC_CACHE;
   parms->ExitResponse = MQXCC_OK;

   switch (parms->ExitReason)
   {	
	case MQXR_INIT:
		memset(parms->ExitUserArea, '0', 1);
		return;
   	case MQXR_CLWL_OPEN:
		index = parms->DestinationChosen - 1;
		break;
	case MQXR_CLWL_PUT:
		memcpy(tmpbuf1, parms->ExitUserArea , 1);	/* pick up prev dest	*/
		index = atol(tmpbuf1);						/* convert to LONG		*/
		break;
	default:
		return;
   }

   if (index < 0) 
			index = 0;									/* check just in case	*/
   if ( index > parms->DestinationCount-1)
			index = 0;
   
   for ( j = 0; j < parms->DestinationCount; j++ )

   {

     GoodDestination = 1;						/* assume all is well	*/
     qmgr = parms->DestinationArrayPtr[ index ]; /* check this qmgr		*/
	 
     MQXCLWLN ( parms, qmgr, qmgr->ChannelDefOffset, &NextRecord, &CompCode, &Reason );

     if ((CompCode == MQCC_OK) && NextRecord)
     {
       cd = NextRecord;							/* address channel info	*/

	   /**************************************************************/
       /* Is the queue PUT enabled ?  Note that the QArrayPtr is     */
       /* NULL in the case of choosing a destination queue manager   */
       /**************************************************************/
        
       if (parms->QArrayPtr)
       {
         queue = parms->QArrayPtr[ index ];

         if (queue && (queue->InhibitPut == MQQA_PUT_INHIBITED))
           GoodDestination = 0;
       }
	
       /**************************************************************/
       /* Don't choose destination qmgrs which have not joined the   */
       /* cluster                                                    */
       /**************************************************************/
	   if ((qmgr->QMgrFlags & MQQMF_AVAILABLE) == 0)
         GoodDestination = 0;
       	
       /**************************************************************/
       /* Only choose destinations with channels in a good state     */
       /**************************************************************/

       switch ( qmgr->ChannelState )
       {
         case MQCHS_INACTIVE:
         case MQCHS_BINDING:
         case MQCHS_STARTING:
         case MQCHS_RUNNING:
         case MQCHS_STOPPING:
         case MQCHS_INITIALIZING:
           break;

         case MQCHS_RETRYING:
         case MQCHS_REQUESTING:
         case MQCHS_PAUSED:
         case MQCHS_STOPPED:
         default:
           GoodDestination = 0;
           break;
       }
	   
       if (GoodDestination)
	   {
		 AtLeastOne = 1;				/* set 'found' flag			*/
		 if (cd->NetworkPriority > Highest)
		 {
			 Highest = cd->NetworkPriority;	/* store highest value	*/
			 Highest_Sel = index+1;			/* store highest entry	*/
		 }

	   }

	 }

	index++;							/* Increment entry			*/
	
	if ( index > parms->DestinationCount-1)
		 index = 0;						/* back to beginning		*/
   }
   
   /* Out of available qmgrs, at least one was eligible				*/
   if (AtLeastOne)
   {
	   if (Highest_Sel > 0)
	     parms->DestinationChosen = Highest_Sel;
	   /* convert destination from LONG to ASCII, and store it		*/
	   _ltoa(parms->DestinationChosen, tmpbuf, 10);
	   memcpy(parms->ExitUserArea , tmpbuf, 1);
   }
   else
	   parms->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
   
   return;
 }


 #if MQAT_DEFAULT == MQAT_UNIX
  void MQStart(){;}
 #endif